@@ -1,10 +1,10 @@
-# $Id: Subscription.pm,v 1.10 2006/06/07 02:42:36 asc Exp $
+# $Id: Subscription.pm,v 1.11 2006/06/17 16:56:37 asc Exp $
 use strict;
 
 package Net::Delicious::Subscription;
 use base qw (Net::Delicious::Object);
 
-$Net::Delicious::Subscription::VERSION = '0.99';
+$Net::Delicious::Subscription::VERSION = '1.0';
 
 =head1 NAME
 
@@ -70,10 +70,7 @@ Returns a string.
 
 =cut
 
-sub user {
-    my $self = shift;
-    return $self->{user};
-}
+# Defined in Net::Delicious::Object
 
 =head2 $obj->tag()
 
@@ -81,10 +78,7 @@ Returns a string.
 
 =cut
 
-sub tag {
-    my $self = shift;
-    return $self->{tag};
-}
+# Defined in Net::Delicious::Object
 
 =head2 $obj->url()
 
@@ -105,18 +99,13 @@ Return the object as a hash ref safe for serializing and re-blessing.
 
 # Defined in Net::Delicious::Object
 
-sub _properties {
-        my $pkg = shift;
-        return qw ( user tag );
-}
-
 =head1 VERSION
 
-0.99
+1.0
 
 =head1 DATE
 
-$Date: 2006/06/07 02:42:36 $
+$Date: 2006/06/17 16:56:37 $
 
 =head1 AUTHOR
 
